import { Coordinate } from '../utils/coordinate';
import { MapElement } from './map-element';
const CONTROL_SIZE = 50;
/**
 * A type of `MapElement`.  A marker is an image or text rendered at a specific
 * point on a Map.
 */
export class MapMarker extends MapElement {
    constructor(opts) {
        super(opts);
        this._anchor = new Coordinate(0, 0);
        this._available = false;
        this._label = '';
        this._opacity = 1;
        this._position = new Coordinate(0, 0);
        if (opts) {
            this.anchor = opts.anchor ? opts.anchor : new Coordinate(0, 0);
            this.available = opts.available ? opts.available : false;
            this.icon = opts.icon ? opts.icon : undefined;
            this.label = (opts.label !== undefined) ? opts.label : '';
            this.opacity = (opts.opacity !== undefined) ? opts.opacity : 1;
            this.position = opts.position;
        }
    }
    /**
     * An offset relative to the `position` of the marker.
     */
    set anchor(newAnchor) { this._anchor = newAnchor; }
    get anchor() { return this._anchor; }
    /**
     * A flag indicating if the this `MapMarker` represents an active or inactive
     * point.  When the marker is available, it gets a matching `--available` CSS
     * class added.
     */
    set available(isAvailable) { this._available = isAvailable; }
    get available() { return this._available; }
    /**
     * The icon that will be drawn for this `MapMarker`.
     */
    set icon(newIcon) { this._icon = newIcon; }
    get icon() { return this._icon; }
    /**
     * A line of text dispayed on the map either on its own as a simple text
     * label or along side the `icon`.
     */
    set label(newLabel) { this._label = newLabel; }
    get label() { return this._label; }
    /**
     * The opacity of the marker, 1 being fully opaque and 0 being fully
     * transparent.
     */
    set opacity(newOpacity) { this._opacity = Math.max(0, Math.min(1, newOpacity)); }
    get opacity() { return this._opacity; }
    /**
     * The overall position of the marker on the `Map`.
     */
    set position(newPosition) { this._position = newPosition; }
    get position() { return this._position; }
    /**
     * Moves this marker by a given amount.
     * @param delta The amount to move by
     */
    move(delta) {
        this._position.translate(delta);
    }
    render() {
        if (!this.visible) {
            return undefined;
        }
        const rectClass = {
            'rl-map-rect': true,
            'rl-map-rect--activated': this.active,
        };
        const contents = [];
        const rect = (h("rect", { class: rectClass, rx: "12", ry: "12", height: CONTROL_SIZE, width: CONTROL_SIZE }));
        if (this._icon === undefined) {
            contents.push(rect);
        }
        else {
            this._icon.size = { width: CONTROL_SIZE, height: CONTROL_SIZE };
            contents.push(this._icon.render());
            contents.push(rect);
        }
        const gTrans = `translate(${this.position.toPathString()})`;
        const gClass = {
            'rl-map-marker': true,
            'rl-map-marker--clickable': this._clickable,
            'rl-map-marker--available': this._available,
        };
        return (h("g", { id: this._id, class: gClass, tabindex: this._clickable ? '0' : undefined, transform: gTrans }, contents));
    }
}
