let nextId = 0;
/**
 * An entry to be displayed as part of a `DetailDialog`.
 */
export class DetailDialogItem {
    constructor() {
        this.index = nextId++;
        this.itemId = 0;
        /**
         * A flag indicating that this item will be removed when all the details are
         * serialized and sent to the server.
         */
        this.remove = false;
        /**
         * The description for this item.
         */
        this.description = '';
        /**
         * The name of this item.
         */
        this.name = '';
        /**
         * Alternate text for this item.
         */
        this.altText = '';
        /**
         * The ID of the detail type of this item.
         */
        this.detailTypeId = -1;
        /**
         * The short code of this item.
         */
        this.code = '';
        /**
         * All the possible values for the type of this `DetailDialogItem`.
         */
        this.typeOptions = [];
    }
    onDetailChanged() {
        if (this.detail) {
            this.description = this.detail.description;
            this.name = this.detail.name;
            this.altText = this.detail.altText;
            this.itemId = this.detail.id;
            this.detailTypeId = this.detail.detailTypeId;
            this.code = this.detail.code;
            if (this.detail.detailTypeId > 0) {
                // Set the selected category and detailType to mathch.
                this.categoryOptions.forEach(c => {
                    c.items.forEach(dt => {
                        if (this.detailTypeId === dt.id) {
                            this.categorySelection = Object.assign({}, c);
                            this.typeSelection = Object.assign({}, dt);
                        }
                    });
                });
            }
        }
    }
    onCategoryOptionsChanged() {
        // if (this.categorySelection === undefined) {
        this.categorySelection = Object.assign({}, this.categoryOptions[0]);
        // }
    }
    onCategorySelectionChanged() {
        if (this.categorySelection !== undefined) {
            this.typeOptions = [...this.categorySelection.items];
            this.typeSelection = Object.assign({}, this.typeOptions[0]);
        }
    }
    onTypeOptionsChanged() {
        // if (this.typeOptions !== undefined) {
        this.typeSelection = this.typeOptions[0];
        // }
    }
    onTypeSelectionChanged() {
        if (this.typeSelection !== undefined) {
            this.detailTypeId = this.typeSelection.id;
        }
    }
    componentDidLoad() {
        this.onDetailChanged();
        this.onCategoryOptionsChanged();
    }
    /**
     * Returns a `Promise` that resolves to a `MapElementDetail` object with
     * values set as those of this `DetailDialogItem`.
     */
    getDetail() {
        return Promise.resolve({
            description: this.description,
            name: this.name,
            code: this.code,
            altText: this.altText,
            id: this.itemId,
            detailTypeId: this.typeSelection ? this.typeSelection.id : 1,
        });
    }
    /**
     * Returns a `Promise` that resolves to whether or not this `DetailDialogItem`
     * is to be removed or not.
     */
    toRemove() {
        return Promise.resolve(this.remove);
    }
    hostData() {
        return {
            class: {
                'rl-detail-dialog-item': true,
                'rl-detail-dialog-item--removed': this.remove,
            },
            id: `rl-detail-dialog-item-${this.index}`
        };
    }
    render() {
        if (this.categorySelection === undefined || this.typeSelection === undefined || this.remove) {
            return null;
        }
        return ([
            h("div", { class: "rl-text-field-container" },
                h("rl-select-menu", { label: "Category", options: this.categoryOptions.map(c => c.name), selectedOption: this.categorySelection.name, onSelected: evt => {
                        this.categorySelection = this.categoryOptions[evt.detail];
                    } })),
            h("div", { class: "rl-text-field-container" },
                h("rl-select-menu", { label: "Type", options: this.typeOptions.map((i) => i.name), selectedOption: this.typeSelection.name, onSelected: evt => {
                        this.typeSelection = this.typeOptions[evt.detail];
                    } })),
            h("div", { class: "rl-text-field-container" },
                h("rl-text-field", { label: "Code", value: this.code, onChangeValue: (e) => this.code = e.detail })),
            h("div", { class: "rl-text-field-container" },
                h("rl-text-field", { label: "Name", value: this.name, onChangeValue: (e) => this.name = e.detail })),
            h("div", { class: "rl-text-field-container" },
                h("rl-text-field", { label: "Description", value: this.description, onChangeValue: (e) => this.description = e.detail })),
            h("div", { class: "rl-text-field-container" },
                h("rl-text-field", { label: "Alt Text", value: this.altText, onChangeValue: (e) => this.altText = e.detail })),
            h("button", { type: "button", class: "mdc-button" },
                h("span", { class: "mdc-button__label", onClick: _ => this.remove = true }, "Remove"))
        ]);
    }
    static get is() { return "rl-detail-dialog-item"; }
    static get properties() { return {
        "altText": {
            "state": true
        },
        "categoryOptions": {
            "type": "Any",
            "attr": "category-options",
            "mutable": true,
            "watchCallbacks": ["onCategoryOptionsChanged"]
        },
        "categorySelection": {
            "type": "Any",
            "attr": "category-selection",
            "mutable": true,
            "watchCallbacks": ["onCategorySelectionChanged"]
        },
        "code": {
            "state": true
        },
        "description": {
            "state": true
        },
        "detail": {
            "type": "Any",
            "attr": "detail",
            "watchCallbacks": ["onDetailChanged"]
        },
        "detailTypeId": {
            "state": true
        },
        "getDetail": {
            "method": true
        },
        "name": {
            "state": true
        },
        "remove": {
            "state": true
        },
        "toRemove": {
            "method": true
        },
        "typeOptions": {
            "type": "Any",
            "attr": "type-options",
            "mutable": true,
            "watchCallbacks": ["onTypeOptionsChanged"]
        },
        "typeSelection": {
            "type": "Any",
            "attr": "type-selection",
            "mutable": true,
            "watchCallbacks": ["onTypeSelectionChanged"]
        }
    }; }
    static get style() { return "/**style-placeholder:rl-detail-dialog-item:**/"; }
}
