import { h } from '../rl-web-components.core.js';

import { a as STATES, b as Coordinate, c as DEFAULT_MAX_SCALE, d as DEFAULT_MIN_SCALE, e as parseElements, f as coordinateFromEvent, j as MapPolygon, g as getTargetId, h as HYSTERESIS, k as MapMarker, l as encodeCoordinates, i as computeLimits } from './chunk-fc3c062f.js';

class RLMapEditor {
    constructor() {
        this.processedElements = [];
        this.initialScale = 1;
        this.state = STATES.NORMAL;
        this.svgScale = 1;
        this.svgTransform = new Coordinate(0, 0);
        this.maxScale = DEFAULT_MAX_SCALE;
        this.minScale = DEFAULT_MIN_SCALE;
    }
    onMapImageChanged() {
        if (this.mapImage === undefined) {
            return;
        }
        const img = new Image();
        img.src = this.mapImage;
        img.onload = () => {
            if (this.imgSize &&
                this.imgSize.width === img.width &&
                this.imgSize.height === img.height) {
                return;
            }
            this.imgSize = new DOMRect(0, 0, img.width, img.height);
            this.computeSvgSize();
            this.computeScale();
            this.updateLimits();
        };
    }
    onElementsChanged() {
        if (this.elements !== undefined) {
            this.processedElements = parseElements(this.elements, this.svgScale);
        }
    }
    componentDidLoad() {
        this.onMapImageChanged();
        this.onElementsChanged();
    }
    onGestureDown(e) {
        this.start = this.last = coordinateFromEvent(e, this.root);
        if (this.state === STATES.NORMAL) {
            this.state = STATES.GESTURE_DOWN;
        }
        if (this.state === STATES.ADD_REGION_INIT) {
            if (this.activeElement && this.activeElement instanceof MapPolygon) {
                this.activeElement.addPoint(this.toSvgSpace(this.start));
                this.root.forceUpdate();
            }
            this.state = STATES.ADD_REGION_FIRST;
        }
        else if (this.state !== STATES.ADD_POINT) {
            const id = getTargetId(e.target);
            if (id !== undefined) {
                this.targetElement = this.processedElements.find(i => i.id === id);
            }
            if (e.target && e.target instanceof SVGCircleElement) {
                if (e.target.classList.contains('rl-map-control') ||
                    e.target.classList.contains('rl-map-midpoint')) {
                    this.targetControl = Number(e.target.getAttribute('index'));
                }
            }
        }
    }
    onGestureMove(e) {
        if (this.state === STATES.NORMAL || this.state === STATES.ADD_REGION_INIT) {
            return;
        }
        const point = coordinateFromEvent(e, this.root);
        const delta = Coordinate.difference(point, this.last);
        let dist = 0;
        if (this.start) {
            switch (this.state) {
                case STATES.GESTURE_DOWN:
                    if (this.targetControl !== undefined || this.activeElement && this.targetElement === this.activeElement) {
                        dist = Coordinate.squareDistance(this.start, point);
                        if (dist > HYSTERESIS) {
                            this.state = STATES.DRAGGING;
                        }
                        break;
                    }
                case STATES.DRAGGING:
                    if (this.targetControl !== undefined && this.activeElement !== undefined && this.activeElement instanceof MapPolygon) {
                        if (this.targetControl % 2 === 1) {
                            const i = Math.floor(this.targetControl / 2) + 1;
                            this.activeElement.addPoint(Coordinate.difference(this.toSvgSpace(point), this.toSvgSpace(this.svgTransform)), i);
                            this.targetControl = i * 2;
                        }
                        else {
                            this.activeElement.movePoint(this.toSvgScale(delta), this.targetControl / 2);
                            this.root.forceUpdate();
                        }
                    }
                    else if (this.targetElement && this.activeElement &&
                        this.activeElement.id === this.targetElement.id) {
                        this.targetElement.move(this.toSvgScale(delta));
                        this.root.forceUpdate();
                    }
                    else {
                        this.state = STATES.DRAGGING;
                        this.svgTransform = Coordinate.sum(this.svgTransform, delta).limit(this.limits).round();
                    }
                    break;
                case STATES.ADD_REGION_FIRST:
                    dist = Coordinate.squareDistance(this.start, point);
                    if (this.activeElement && this.activeElement instanceof MapPolygon && dist > HYSTERESIS) {
                        this.state = STATES.ADD_REGION;
                        this.activeElement.addPoint(this.toSvgSpace(point));
                        this.root.forceUpdate();
                    }
                    break;
                case STATES.ADD_REGION:
                    if (this.activeElement && this.activeElement instanceof MapPolygon) {
                        const idx = this.activeElement.points.length - 1;
                        this.activeElement.movePoint(this.toSvgScale(delta), idx);
                        this.root.forceUpdate();
                    }
                    break;
                default:
            }
        }
        this.last = point;
    }
    onGestureUp(e) {
        const point = coordinateFromEvent(e, this.root);
        switch (this.state) {
            case STATES.GESTURE_DOWN:
                if (this.targetElement) {
                    if (this.targetElement !== this.activeElement) {
                        this._setActiveElement(this.targetElement);
                    }
                }
                else {
                    this._clearActiveElement();
                    this.elementDeselected.emit();
                }
                this.state = STATES.NORMAL;
                break;
            case STATES.DRAGGING:
                const delta = Coordinate.difference(point, this.last);
                if (this.activeElement && this.activeElement instanceof MapPolygon && this.targetControl !== undefined) {
                    this.activeElement.movePoint(this.toSvgScale(delta), this.targetControl / 2);
                    this.elementUpdated.emit(this.mapElementFromParsedElement(this.activeElement));
                }
                else if (this.targetElement && this.activeElement &&
                    this.activeElement.id === this.targetElement.id) {
                    this.activeElement.move(this.toSvgScale(delta));
                    this.elementUpdated.emit(this.mapElementFromParsedElement(this.activeElement));
                }
                else {
                    this.svgTransform = Coordinate.sum(this.svgTransform, delta).limit(this.limits).round();
                }
                this.state = STATES.NORMAL;
                break;
            case STATES.ADD_REGION_FIRST:
                if (this.activeElement && this.activeElement instanceof MapPolygon) {
                    this.activeElement.addPoint(this.toSvgSpace(point));
                }
                this.state = STATES.ADD_REGION;
                break;
            case STATES.ADD_REGION:
                if (this.activeElement && this.activeElement instanceof MapPolygon) {
                    if (this.targetControl === 0) {
                        this.activeElement.removePoint();
                        this.state = STATES.NORMAL;
                        this.root.forceUpdate();
                        this.elementCreated.emit(this.mapElementFromParsedElement(this.activeElement));
                    }
                    else {
                        this.activeElement.addPoint(this.toSvgSpace(point));
                    }
                }
                break;
            case STATES.ADD_POINT:
                if (this.activeElement && this.activeElement instanceof MapMarker && this.root) {
                    this.activeElement.position = this.toSvgSpace(point);
                    this.state = STATES.NORMAL;
                    this.root.forceUpdate();
                    this.elementCreated.emit(this.mapElementFromParsedElement(this.activeElement));
                }
            default:
        }
        this.targetControl = undefined;
        this.targetElement = undefined;
    }
    onMouseLeave(e) {
        if (this.state === STATES.DRAGGING ||
            this.state === STATES.GESTURE_DOWN) {
            this.state = STATES.NORMAL;
            const point = coordinateFromEvent(e, this.root);
            if (this.state === STATES.DRAGGING && this.activeElement) {
                const delta = Coordinate.difference(point, this.last);
                if (this.activeElement && this.activeElement instanceof MapPolygon && this.targetControl !== undefined) {
                    this.activeElement.movePoint(this.toSvgScale(delta), this.targetControl);
                    this.elementUpdated.emit(this.mapElementFromParsedElement(this.activeElement));
                }
                else if (this.targetElement && this.activeElement &&
                    this.activeElement.id === this.targetElement.id) {
                    this.activeElement.move(this.toSvgScale(delta));
                    this.elementUpdated.emit(this.mapElementFromParsedElement(this.activeElement));
                }
                else {
                    this.svgTransform = Coordinate.sum(this.svgTransform, delta).limit(this.limits).round();
                }
                this.state = STATES.NORMAL;
            }
        }
    }
    onDoubleClick(e) {
        if (this.state === STATES.NORMAL) {
            const id = getTargetId(e.target);
            if (id !== undefined && this.elements !== undefined) {
                this.elementDoubleClicked.emit(this.elements[id]);
            }
        }
    }
    onResize() {
        clearTimeout(this.resizeDebounce);
        this.resizeDebounce = setTimeout(_ => {
            this.computeSvgSize();
            this.computeScale();
            this.updateLimits();
            this.svgTransform = this.svgTransform.clone().limit(this.limits);
        }, 150);
    }
    onWheel(e) {
        const oldPos = this.toSvgSpace(new Coordinate(e.clientX, e.clientY));
        if (e.deltaY < 0) {
            if (this.svgScale * 1.05263157895 < this.initialScale * this.maxScale) {
                this.svgScale *= 1.05263157895;
            }
        }
        else {
            if (this.svgScale * 0.95 > this.initialScale * this.minScale) {
                this.svgScale *= 0.95;
            }
        }
        this.updateLimits();
        this.processedElements.forEach(el => el.scale = this.svgScale);
        const newPos = this.toSvgSpace(new Coordinate(e.clientX, e.clientY));
        const delta = Coordinate.difference(newPos, oldPos).scale(this.svgScale);
        this.svgTransform.translate(delta).limit(this.limits).round();
    }
    onEnter(e) {
        if (e.key === 'enter' && e.target && e.target instanceof SVGElement &&
            e.target.classList.contains('rl-map-element')) {
            const id = Number(e.target.id);
            const el = this.processedElements.find(i => i.id === id);
            this._setActiveElement(el);
        }
    }
    addRegion() {
        this._clearActiveElement();
        this.state = STATES.ADD_REGION_INIT;
        this.activeElement = new MapPolygon();
        this.activeElement.open = true;
        this.activeElement.scale = this.svgScale;
        this.processedElements = [...this.processedElements, this.activeElement];
    }
    addPoint() {
        this._clearActiveElement();
        this.state = STATES.ADD_POINT;
        this.activeElement = new MapMarker();
        this.processedElements = [...this.processedElements, this.activeElement];
    }
    cancelAction() {
        this._clearActiveElement();
        this.state = STATES.NORMAL;
    }
    deleteRegion() {
        if (this.elements !== undefined && this.state === STATES.NORMAL &&
            this.activeElement) {
            this.elementDeleted.emit(this.elements[this.activeElement.id]);
            this._clearActiveElement();
        }
    }
    setActiveElement(id) {
        if (this.processedElements.length > 0) {
            this._setActiveElement(this.processedElements.find(i => i.id === id));
        }
    }
    mapElementFromParsedElement(el) {
        if (this.elements === undefined) {
            return undefined;
        }
        const originalEl = this.elements[el.id] || {
            altText: '',
            description: '',
            details: undefined,
            floorId: 0,
            iconPath: [],
            enabled: true,
        };
        return {
            altText: originalEl.altText,
            description: originalEl.description,
            details: originalEl.details,
            enabled: originalEl.enabled,
            floorId: originalEl.floorId,
            id: el.id,
            icon: originalEl.icon,
            symbol: originalEl.symbol,
            name: el.name,
            points: el instanceof MapPolygon ? encodeCoordinates(el.points) : encodeCoordinates(el.position),
        };
    }
    _clearActiveElement() {
        if (this.activeElement) {
            this.activeElement.active = false;
        }
        this.activeElement = undefined;
    }
    updateLimits() {
        this.limits = computeLimits(this.imgSize, this.svgSize, this.svgScale);
    }
    computeScale() {
        const img = this.imgSize;
        const svg = this.svgSize;
        this.svgScale = this.initialScale =
            Math.max(svg.width / img.width, svg.height / img.height);
        this.processedElements.forEach(el => el.scale = this.svgScale);
    }
    computeSvgSize() {
        const svg = this.root.querySelector('svg');
        if (svg) {
            const rect = svg.getBoundingClientRect();
            this.svgSize = new DOMRect(0, 0, rect.width, rect.height);
        }
    }
    _setActiveElement(el) {
        if (!el) {
            this._clearActiveElement();
            this.elementDeselected.emit();
            return;
        }
        this._clearActiveElement();
        this.activeElement = el;
        this.activeElement.active = true;
        if (this.elements !== undefined) {
            this.elementSelected.emit(this.elements[this.activeElement.id]);
        }
    }
    toSvgScale(c) {
        return c.clone().scale(1 / this.svgScale).round();
    }
    toSvgSpace(c) {
        return Coordinate.difference(this.toSvgScale(c), this.toSvgScale(this.svgTransform));
    }
    hostData() {
        return {
            class: {
                'rl-map': true,
                'rl-map-editor': true,
            },
        };
    }
    render() {
        const m = [
            this.svgScale, 0, 0, this.svgScale,
            this.svgTransform.x,
            this.svgTransform.y,
        ];
        const matrix = 'matrix(' + m.join(',') + ')';
        return (h("svg", { class: "rl-map__svg" },
            h("g", { class: "rl-map__transform", transform: matrix },
                h("g", { class: "rl-map__image-wrapper" },
                    h("image", { class: "rl-map__image", xlinkHref: this.mapImage !== undefined ? this.mapImage : undefined })),
                h("g", { class: "rl-map__elements" }, this.processedElements.map(el => el.render())),
                this.activeElement && this.activeElement instanceof MapPolygon && this.activeElement.renderControls())));
    }
    static get is() { return "rl-map-editor"; }
    static get properties() { return {
        "activeElement": {
            "state": true
        },
        "addPoint": {
            "method": true
        },
        "addRegion": {
            "method": true
        },
        "cancelAction": {
            "method": true
        },
        "deleteRegion": {
            "method": true
        },
        "elements": {
            "type": "Any",
            "attr": "elements",
            "mutable": true,
            "watchCallbacks": ["onElementsChanged"]
        },
        "mapImage": {
            "type": String,
            "attr": "map-image",
            "watchCallbacks": ["onMapImageChanged"]
        },
        "maxScale": {
            "type": Number,
            "attr": "max-scale"
        },
        "minScale": {
            "type": Number,
            "attr": "min-scale"
        },
        "root": {
            "elementRef": true
        },
        "setActiveElement": {
            "method": true
        },
        "svgScale": {
            "state": true
        },
        "svgTransform": {
            "state": true
        }
    }; }
    static get events() { return [{
            "name": "elementSelected",
            "method": "elementSelected",
            "bubbles": true,
            "cancelable": true,
            "composed": true
        }, {
            "name": "elementUpdated",
            "method": "elementUpdated",
            "bubbles": true,
            "cancelable": true,
            "composed": true
        }, {
            "name": "elementCreated",
            "method": "elementCreated",
            "bubbles": true,
            "cancelable": true,
            "composed": true
        }, {
            "name": "elementDeselected",
            "method": "elementDeselected",
            "bubbles": true,
            "cancelable": true,
            "composed": true
        }, {
            "name": "elementDeleted",
            "method": "elementDeleted",
            "bubbles": true,
            "cancelable": true,
            "composed": true
        }, {
            "name": "elementDoubleClicked",
            "method": "elementDoubleClicked",
            "bubbles": true,
            "cancelable": true,
            "composed": true
        }]; }
    static get listeners() { return [{
            "name": "mousedown",
            "method": "onGestureDown",
            "passive": true
        }, {
            "name": "touchstart",
            "method": "onGestureDown",
            "passive": true
        }, {
            "name": "mousemove",
            "method": "onGestureMove",
            "passive": true
        }, {
            "name": "touchmove",
            "method": "onGestureMove",
            "passive": true
        }, {
            "name": "mouseup",
            "method": "onGestureUp",
            "passive": true
        }, {
            "name": "touchend",
            "method": "onGestureUp",
            "passive": true
        }, {
            "name": "mouseleave",
            "method": "onMouseLeave",
            "passive": true
        }, {
            "name": "dblclick",
            "method": "onDoubleClick"
        }, {
            "name": "window:resize",
            "method": "onResize",
            "passive": true
        }, {
            "name": "wheel",
            "method": "onWheel",
            "passive": true
        }, {
            "name": "keydown",
            "method": "onEnter"
        }]; }
    static get style() { return ".rl-map{display:inline-block;overflow:hidden}.rl-map,.rl-map__svg{width:100%;height:100%}.rl-map-region{fill:var(--rl-map-region-fill,rgba(0,76,155,.6));stroke:var(--rl-map-region-fill,#004c9b);stroke-width:4}.rl-map-region:focus{outline:none;fill:var(--rl-map-element-focus-fill,rgba(255,255,83,.3))}.rl-map-region--activated{stroke:var(--rl-map-element-active-stroke,#c7ab00);stroke-width:8}.rl-map-region:hover{outline:none;fill:var(--rl-map-element-focus-fill,rgba(255,255,83,.3));cursor:pointer}.rl-map-text{font-family:var(--rl-map-element-font-family,\"sans\");font-size:var(--rl-map-element-font-size,24px);-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none}.rl-map-text:hover{cursor:pointer}.rl-map-outline{fill:none;stroke:var(--rl-map-element-active-stroke,#c7ab00);stroke-width:2}.rl-map-control{stroke:#000;fill:hsla(0,0%,100%,.6);stroke-width:2}.rl-map-control:hover{fill:hsla(0,0%,75.3%,.6);cursor:pointer}.rl-map-midpoint{stroke:rgba(0,0,0,.6);fill:hsla(0,0%,100%,.3);stroke-width:2}.rl-map-midpoint:hover{fill:hsla(0,0%,75.3%,.3);cursor:pointer}.rl-map-point:focus{outline:none}.rl-map-point:hover rect{outline:none;fill:var(--rl-map-element-focus-fill,rgba(255,255,83,.3));cursor:pointer}.rl-map-rect{fill:var(--rl-map-region-fill,rgba(0,76,155,.6));stroke:var(--rl-map-region-fill,#004c9b);stroke-width:4}.rl-map-rect--activated{stroke:var(--rl-map-element-active-stroke,#c7ab00);stroke-width:8;outline:none;fill:var(--rl-map-element-focus-fill,rgba(255,255,83,.3))}"; }
}

export { RLMapEditor as RlMapEditor };
