import { h } from '../rl-web-components.core.js';

class Coordinate {
    constructor(x, y) {
        this.x = x;
        this.y = y;
    }
    clone() {
        return new Coordinate(this.x, this.y);
    }
    toString() {
        return '(' + this.x + ', ' + this.y + ')';
    }
    toPathString() {
        return this.x + ' ' + this.y;
    }
    equals(other) {
        return other instanceof Coordinate && Coordinate.equals(this, other);
    }
    ceil() {
        this.x = Math.ceil(this.x);
        this.y = Math.ceil(this.y);
        return this;
    }
    floor() {
        this.x = Math.floor(this.x);
        this.y = Math.floor(this.y);
        return this;
    }
    translate(tx, ty) {
        if (tx instanceof Coordinate) {
            this.x += tx.x;
            this.y += tx.y;
        }
        else {
            this.x += Number(tx);
            if (typeof ty === 'number') {
                this.y += ty;
            }
        }
        return this;
    }
    scale(sx, sy) {
        sy = typeof sy === 'number' ? sy : sx;
        this.x *= sx;
        this.y *= sy;
        return this;
    }
    rotateRadians(radians, center) {
        center = center || new Coordinate(0, 0);
        const x = this.x;
        const y = this.y;
        const cos = Math.cos(radians);
        const sin = Math.sin(radians);
        this.x = (x - center.x) * cos - (y - center.y) * sin + center.x;
        this.y = (x - center.x) * sin + (y - center.y) * cos + center.y;
        return this;
    }
    rotateDegrees(degrees, center) {
        return this.rotateRadians(degrees * Math.PI / 180, center);
    }
    limit(limits) {
        if (!limits) {
            throw new Error('Unable to limit Corrdinate, limits undefined!');
        }
        const left = !isNaN(limits.left) ? limits.left : undefined;
        const width = !isNaN(limits.width) ? limits.width : 0;
        const maxX = left !== undefined ? left + width : Infinity;
        const minX = left !== undefined ? left : -Infinity;
        this.x = Math.min(maxX, Math.max(minX, this.x));
        const top = !isNaN(limits.top) ? limits.top : undefined;
        const height = !isNaN(limits.height) ? limits.height : 0;
        const maxY = top !== undefined ? top + height : Infinity;
        const minY = top !== undefined ? top : -Infinity;
        this.y = Math.min(maxY, Math.max(minY, this.y));
        return this;
    }
    round(precision = 0) {
        const factor = Math.pow(10, precision);
        this.x = Math.round(this.x * factor) / factor;
        this.y = Math.round(this.y * factor) / factor;
        return this;
    }
    static equals(a, b) {
        if (a === b) {
            return true;
        }
        return a.x === b.x && a.y === b.y;
    }
    static distance(a, b) {
        const dx = a.x - b.x;
        const dy = a.y - b.y;
        return Math.sqrt(dx * dx + dy * dy);
    }
    static magnitude(a) {
        return Math.sqrt(a.x * a.x + a.y * a.y);
    }
    static squareDistance(a, b) {
        const dx = a.x - b.x;
        const dy = a.y - b.y;
        return dx * dx + dy * dy;
    }
    static difference(a, b) {
        if (!b) {
            return new Coordinate(0, 0);
        }
        return new Coordinate(a.x - b.x, a.y - b.y);
    }
    static sum(a, b) {
        return new Coordinate(a.x + b.x, a.y + b.y);
    }
    static midpoint(a, b) {
        return new Coordinate((a.x + b.x) / 2, (a.y + b.y) / 2);
    }
}

function getClientPosition(el) {
    if (el.nodeType === 1) {
        return getClientPositionForElement_(el);
    }
    else {
        const targetEvent = el.changedTouches ? el.changedTouches[0] : el;
        return new Coordinate(targetEvent.clientX, targetEvent.clientY);
    }
}
function getClientPositionForElement_(el) {
    const box = getBoundingClientRect_(el);
    return new Coordinate(box.left, box.top);
}
function getBoundingClientRect_(el) {
    let rect;
    try {
        rect = el.getBoundingClientRect();
    }
    catch (e) {
        return { 'left': 0, 'top': 0, 'right': 0, 'bottom': 0 };
    }
    return rect;
}
function coordinateFromEvent(a, b) {
    const ap = getClientPosition(a);
    const bp = getClientPosition(b);
    return new Coordinate(ap.x - bp.x, ap.y - bp.y);
}
function decodeCoordinates(data, closed) {
    const pointData = data.split(' ');
    let points = [];
    let i = pointData.length - 2;
    let x;
    let y;
    let path = closed ? 'Z' : '';
    while (i >= 0) {
        x = Math.round(Number(pointData[i]));
        y = Math.round(Number(pointData[i + 1]));
        points.push(new Coordinate(x, y));
        path = (i === 0 ? ['M', x, y, path].join(' ') : ['L', x, y, path].join(' '));
        i -= 2;
    }
    if (points.length === 1) {
        points = points[0];
    }
    return { path, points };
}
function encodeCoordinates(points) {
    if (points instanceof Coordinate) {
        return points.toPathString();
    }
    else {
        return points.map(i => {
            return i.toPathString();
        }).join(' ');
    }
}
function pathFromCoordinateArray(points, open = false) {
    return points.map((i, idx) => {
        if (idx === 0) {
            return 'M ' + i.toPathString();
        }
        else if (idx === points.length - 1 && !open) {
            return ' L ' + i.toPathString() + ' Z';
        }
        else {
            return ' L ' + i.toPathString();
        }
    }).join('');
}

class MapElement {
    constructor(opts) {
        this._active = false;
        this._clickable = true;
        this._id = -1;
        this._name = '';
        this._scale = 1;
        this._visible = true;
        this._zIndex = 1;
        if (opts) {
            this.name = (opts.name !== undefined) ? opts.name : '';
            this.zIndex = (opts.zIndex !== undefined) ? opts.zIndex : 1;
        }
    }
    set active(isActive) { this._active = isActive; }
    get active() { return this._active; }
    set clickable(isActive) { this._clickable = isActive; }
    get clickable() { return this._clickable; }
    set id(newId) { this._id = newId; }
    get id() { return this._id; }
    set name(newName) { this._name = newName; }
    get name() { return this._name; }
    set scale(newScale) { this._scale = newScale; }
    get scale() { return this._scale; }
    set visible(isVisible) { this._visible = isVisible; }
    get visible() { return this._visible; }
    set zIndex(isVisible) { this._zIndex = isVisible; }
    get zIndex() { return this._zIndex; }
}

const CONTROL_SIZE = 50;
class MapMarker extends MapElement {
    constructor(opts) {
        super(opts);
        this._anchor = new Coordinate(0, 0);
        this._available = false;
        this._label = '';
        this._opacity = 1;
        this._position = new Coordinate(0, 0);
        if (opts) {
            this.anchor = opts.anchor ? opts.anchor : new Coordinate(0, 0);
            this.available = opts.available ? opts.available : false;
            this.icon = opts.icon ? opts.icon : undefined;
            this.label = (opts.label !== undefined) ? opts.label : '';
            this.opacity = (opts.opacity !== undefined) ? opts.opacity : 1;
            this.position = opts.position;
        }
    }
    set anchor(newAnchor) { this._anchor = newAnchor; }
    get anchor() { return this._anchor; }
    set available(isAvailable) { this._available = isAvailable; }
    get available() { return this._available; }
    set icon(newIcon) { this._icon = newIcon; }
    get icon() { return this._icon; }
    set label(newLabel) { this._label = newLabel; }
    get label() { return this._label; }
    set opacity(newOpacity) { this._opacity = Math.max(0, Math.min(1, newOpacity)); }
    get opacity() { return this._opacity; }
    set position(newPosition) { this._position = newPosition; }
    get position() { return this._position; }
    move(delta) {
        this._position.translate(delta);
    }
    render() {
        if (!this.visible) {
            return undefined;
        }
        const rectClass = {
            'rl-map-rect': true,
            'rl-map-rect--activated': this.active,
        };
        const contents = [];
        const rect = (h("rect", { class: rectClass, rx: "12", ry: "12", height: CONTROL_SIZE, width: CONTROL_SIZE }));
        if (this._icon === undefined) {
            contents.push(rect);
        }
        else {
            this._icon.size = { width: CONTROL_SIZE, height: CONTROL_SIZE };
            contents.push(this._icon.render());
            contents.push(rect);
        }
        const gTrans = `translate(${this.position.toPathString()})`;
        const gClass = {
            'rl-map-marker': true,
            'rl-map-marker--clickable': this._clickable,
            'rl-map-marker--available': this._available,
        };
        return (h("g", { id: this._id, class: gClass, tabindex: this._clickable ? '0' : undefined, transform: gTrans }, contents));
    }
}

class MapPolygon extends MapElement {
    constructor() {
        super(...arguments);
        this._open = false;
        this._path = '';
        this._points = [];
    }
    set open(open) {
        this._open = open;
        this._path = pathFromCoordinateArray(this._points, open);
    }
    get open() { return this._open; }
    set points(newPoint) {
        this._points = newPoint;
        this._path = pathFromCoordinateArray(newPoint, this._open);
    }
    get points() { return this._points; }
    addPoint(c, index) {
        const pts = this._points;
        this.points = index !== undefined ?
            [...pts.slice(0, index), c, ...pts.slice(index)] :
            [...pts, c];
    }
    move(delta) {
        const p = this.points;
        p.forEach(c => c.translate(delta));
        this.points = [...p];
    }
    movePoint(delta, index) {
        const p = this.points;
        p[index].translate(delta);
        this.points = [...p];
    }
    removePoint(index) {
        const p = this._points;
        if (index !== undefined) {
            p.splice(index, 1);
        }
        else {
            p.splice(p.length - 1, 1);
        }
        this.points = [...p];
    }
    render() {
        if (this._visible === false) {
            return undefined;
        }
        const regionClass = {
            'rl-map-polygon': true,
            'rl-map-polygon--activated': this._active,
        };
        return (h("path", { id: this._id, class: regionClass, d: this._path, tabindex: "0", "aria-label": this._name, role: "button" }));
    }
    renderControls() {
        const pts = this._points;
        if (this._visible === false || pts.length === 0) {
            return undefined;
        }
        const points = pts.filter((_, idx) => (pts.length === 1 || (this._open && idx === pts.length - 1 ? false : true))).map((i, index) => {
            const controls = [(h("circle", { cx: i.x, cy: i.y, r: 8 / this._scale, index: index * 2, class: "rl-map-control" }))];
            if (!this._open) {
                const mid = (index === pts.length - 1) ?
                    Coordinate.midpoint(i, pts[0]) :
                    Coordinate.midpoint(i, pts[index + 1]);
                controls.push((h("circle", { cx: mid.x, cy: mid.y, r: 8 / this._scale, index: index * 2 + 1, class: "rl-map-midpoint" })));
            }
            return controls;
        });
        const path = (h("path", { class: "rl-map-outline", d: pathFromCoordinateArray(pts, this._open) }));
        return (h("g", null,
            path,
            points));
    }
}

class MarkerIcon {
    constructor() {
        this._anchor = new Coordinate(0, 0);
        this._originalSize = { width: 1, height: 1 };
        this._size = { width: 1, height: 1 };
        this._scale = 1;
        this._url = '';
    }
    set anchor(c) { this._anchor = c; }
    get anchor() { return this._anchor; }
    set originalSize(s) { this._originalSize = s; }
    get originalSize() { return this._originalSize; }
    set size(s) { this._size = s; }
    get size() { return this._size; }
    set scale(s) { this._scale = s; }
    get scale() { return this._scale; }
    set url(url) { this._url = url; }
    get url() { return this._url; }
    render() {
        const t = `translate(${this._anchor.toPathString()}) scale(${this._scale})`;
        return (h("image", { class: "rl-map-element__icon", height: this.size.height, transform: t, xlinkHref: this._url }));
    }
}

var MarkerSymbolPaths;
(function (MarkerSymbolPaths) {
    MarkerSymbolPaths["computer"] = "M 8 36 c -2 0 -4 -2 -4 -4 v -20 c 0 -2 2 -4 4 -4 h 32 c 2 0 4 2 4 4 v 20 c 0 2 -2 4 -4 4 h 8 v 4 h -48 v -4 z M 8 12 h 32 v 20 h -32 z";
})(MarkerSymbolPaths || (MarkerSymbolPaths = {}));
class MarkerSymbol {
    constructor() {
        this._anchor = new Coordinate(0, 0);
        this._path = '';
        this._pathName = '';
        this._rotation = 0;
        this._size = { width: 1, height: 1 };
    }
    set anchor(c) { this._anchor = c; }
    get anchor() { return this._anchor; }
    set path(p) {
        if (p in MarkerSymbolPaths) {
            this._pathName = p;
            this._path = MarkerSymbolPaths[p];
        }
        else {
            this._pathName = 'custom';
            this._path = p;
        }
    }
    get path() { return this._path; }
    set rotation(r) { this._rotation = r % 360; }
    get rotation() { return this._rotation; }
    set size(s) { this._size = s; }
    get size() { return this._size; }
    render() {
        const sx = this._size.width / 48;
        const sy = this._size.height / 48;
        const tr = this._anchor.toPathString();
        const ro = this._rotation;
        const t = `translate(${tr}) rotate(${ro}) scale(${sx}, ${sy})`;
        const pathClass = {
            'rl-map-symbol': true,
        };
        pathClass[`rl-map-symbol--${this._pathName}`] = true;
        return (h("path", { class: pathClass, transform: t, d: this._path }));
    }
}

const HYSTERESIS = 8;
const DEFAULT_MAX_SCALE = 3;
const DEFAULT_MIN_SCALE = 1;
const STATES = {
    NORMAL: 0,
    GESTURE_DOWN: 1,
    DRAGGING: 2,
    ZOOMING: 3,
    ADD_REGION_INIT: 10,
    ADD_REGION_FIRST: 11,
    ADD_REGION: 12,
    ADD_POINT: 20,
    DRAG_ELEMENT: 30,
};
function computeLimits(iSize, sSize, scale) {
    return new DOMRect(Math.min(0, sSize.width - iSize.width * scale), Math.min(0, sSize.height - iSize.height * scale), Math.abs(iSize.width * scale - sSize.width), Math.abs(iSize.height * scale - sSize.height));
}
function getTargetId(el) {
    if (el instanceof SVGImageElement || el instanceof SVGRectElement) {
        el = el.parentElement;
    }
    if (el instanceof SVGPathElement || el instanceof SVGGElement) {
        return Number(el.id);
    }
    return undefined;
}
function parseElements(elements, scale = 1) {
    return Object.values(elements).map((el) => {
        const decoded = decodeCoordinates(el.points, true);
        let icon;
        if (el.icon !== undefined) {
            icon = new MarkerIcon();
            icon.url = el.icon;
        }
        else if (el.symbol !== undefined && el.symbol in MarkerSymbolPaths) {
            icon = new MarkerSymbol();
            icon.path = el.symbol;
        }
        let newEl;
        if (decoded.points instanceof Coordinate) {
            newEl = new MapMarker();
            newEl.icon = icon;
            newEl.position = decoded.points;
            newEl.available = el.available || false;
        }
        else {
            newEl = new MapPolygon();
            newEl.points = decoded.points;
        }
        newEl.id = el.id;
        newEl.name = el.name;
        newEl.visible = el.enabled;
        newEl.scale = scale;
        newEl.clickable = el.clickable !== undefined ? el.clickable : true;
        return newEl;
    });
}

export { STATES as a, Coordinate as b, DEFAULT_MAX_SCALE as c, DEFAULT_MIN_SCALE as d, parseElements as e, coordinateFromEvent as f, getTargetId as g, HYSTERESIS as h, computeLimits as i, MapPolygon as j, MapMarker as k, encodeCoordinates as l };
