const h = window.RlWebComponents.h;

import { a as STATES, b as Coordinate, c as DEFAULT_MAX_SCALE, d as DEFAULT_MIN_SCALE, e as parseElements, f as coordinateFromEvent, g as getTargetId, h as HYSTERESIS, i as computeLimits } from './chunk-b5c36cc8.js';

class RLMap {
    constructor() {
        /**
         * The array of MapElements currently being displayed.  Created from the
         * `elements` Prop with additional internal information added.
         */
        this.processedElements = [];
        // The original scale factor used to size the SVG so it fits.
        this.initialScale = 1;
        // The current state of the Map.
        this.state = STATES.NORMAL;
        /**
         * The factor by which the Map contents are changed to fit within the SVG.
         */
        this.svgScale = 1;
        /**
         * The x,y translation applied to move the map contents within the bounds of
         * the SVG.
         */
        this.svgTransform = new Coordinate(0, 0);
        /**
         *  The maximum scale factor.
         */
        this.maxScale = DEFAULT_MAX_SCALE;
        /**
         * The minimum scale factor.
         */
        this.minScale = DEFAULT_MIN_SCALE;
    }
    /**
     * Handles when the map Image is loaded/updated.
     */
    onMapImageChanged() {
        if (this.mapImage === undefined) {
            return;
        }
        const img = new Image();
        img.src = this.mapImage;
        img.onload = () => {
            if (this.imgSize &&
                this.imgSize.width === img.width &&
                this.imgSize.height === img.height) {
                return;
            }
            this.imgSize = new DOMRect(0, 0, img.width, img.height);
            this.computeSvgSize();
            this.computeScale();
            this.computeLimits();
        };
    }
    /**
     * Handle when the list of specified elements changes.
     */
    onElementsChanged() {
        this.processedElements = parseElements(this.elements);
    }
    componentDidLoad() {
        this.onMapImageChanged();
        this.onElementsChanged();
        this.onResize();
    }
    componentDidUpdate() {
        this.mapRendered.emit();
    }
    /**
     * Handles when the user triggers a touch/down event to start interaction with
     * the map.
     * @param e The triggering event.
     */
    onGestureDown(e) {
        this.start = this.last = coordinateFromEvent(e, this.root);
        if (this.state === STATES.NORMAL) {
            this.state = STATES.GESTURE_DOWN;
        }
        // Get the ID of the event target, if it is the correct type.
        const id = getTargetId(e.target);
        if (id !== undefined) {
            this.targetElement = this.processedElements.find(i => i.id === id);
        }
    }
    /**
     * Handles when the user triggers a movement event.
     * @param e The triggering event.
     */
    onGestureMove(e) {
        if (this.state === STATES.NORMAL) {
            return;
        }
        // Convert the coordinate to SVG coordinate space.
        const point = coordinateFromEvent(e, this.root);
        switch (this.state) {
            case STATES.GESTURE_DOWN:
                const dist = Coordinate.squareDistance(this.start, point);
                if (dist > HYSTERESIS) {
                    this.state = STATES.DRAGGING;
                }
                break;
            case STATES.DRAGGING:
                const delta = Coordinate.difference(point, this.last);
                this.svgTransform = Coordinate.sum(this.svgTransform, delta).limit(this.limits).round();
                break;
            default:
        }
        this.last = point;
    }
    /**
     * Handles when the user ends their interaction with the Map.
     * @param e The triggering event.
     */
    onGestureUp(e) {
        const point = coordinateFromEvent(e, this.root);
        if (this.state === STATES.DRAGGING) {
            const delta = Coordinate.difference(point, this.last);
            this.svgTransform = Coordinate.sum(this.svgTransform, delta).limit(this.limits).round();
            this.state = STATES.NORMAL;
        }
        else if (this.state === STATES.GESTURE_DOWN) {
            if (this.targetElement) {
                if (this.targetElement !== this.activeElement && this.targetElement.clickable) {
                    this._setActiveElement(this.targetElement);
                    this.targetElement = undefined;
                }
            }
            else {
                if (this.activeElement) {
                    this.elementDeselected.emit(this.activeElement);
                    this._clearActiveElement();
                }
            }
            this.state = STATES.NORMAL;
        }
    }
    /**
     * Handles when the user leaves the bounds of the Map entirely.
     * @param e The triggering event.
     */
    onMouseLeave(e) {
        if (this.state === STATES.DRAGGING ||
            this.state === STATES.GESTURE_DOWN) {
            this.state = STATES.NORMAL;
            const point = coordinateFromEvent(e, this.root);
            if (this.state === STATES.DRAGGING) {
                const delta = Coordinate.difference(point, this.last);
                this.svgTransform = Coordinate.sum(this.svgTransform, delta).limit(this.limits).round();
                this.state = STATES.NORMAL;
            }
        }
    }
    /**
     * Handles when the user performs a resize action.
     */
    onResize() {
        clearTimeout(this.resizeDebounce);
        this.resizeDebounce = setTimeout(_ => {
            // Use a 150ms delay to debounce multiple resize calls and effectively
            // perform the resize action only when the user is 'done' resizing.
            this.computeSvgSize();
            this.computeScale();
            this.computeLimits();
            this.svgTransform = this.svgTransform.clone().limit(this.limits);
        }, 150);
    }
    /**
     * Handles when the user performs a mouse wheen event.  The Map will be zoomed
     * in or out depending on the direction of the wheel.
     *
     * @param e The triggering event.
     */
    onWheel(e) {
        const oldPos = this.toSvgSpace(new Coordinate(e.clientX, e.clientY));
        if (e.deltaY < 0) {
            if (this.svgScale * 1.05263157895 < this.initialScale * this.maxScale) {
                this.svgScale *= 1.05263157895;
            }
        }
        else {
            if (this.svgScale * 0.95 > this.initialScale * this.minScale) {
                this.svgScale *= 0.95;
            }
        }
        this.computeLimits();
        const newPos = this.toSvgSpace(new Coordinate(e.clientX, e.clientY));
        const delta = Coordinate.difference(newPos, oldPos).scale(this.svgScale);
        this.svgTransform.translate(delta).limit(this.limits).round();
    }
    onEnter(e) {
        if (e.key === 'enter' && e.target && e.target instanceof SVGElement &&
            (e.target.classList.contains('rl-map-polygon') ||
                e.target.classList.contains('rl-map-marker'))) {
            const id = Number(e.target.id);
            const el = this.processedElements.find(i => i.id === id);
            this._setActiveElement(el);
        }
    }
    /**
     * Clears the currently active element.
     */
    clearActiveElement() {
        this._clearActiveElement();
    }
    /**
     * Sets the element with the specified ID to active.
     *
     * @param id The ID of the element to set as active.
     */
    setActiveElement(id) {
        if (this.processedElements.length > 0) {
            this._setActiveElement(this.processedElements.find(i => i.id === id), false);
        }
    }
    /**
     * Clears the currently active element.
     */
    _clearActiveElement() {
        if (this.activeElement) {
            this.activeElement.active = false;
        }
        this.activeElement = undefined;
    }
    /**
     * Updates the limits used to restrict the how far the SVG transform can be
     * moved keeping the map from being panning beyond the edge of the image.
     */
    computeLimits() {
        this.limits = computeLimits(this.imgSize, this.svgSize, this.svgScale);
    }
    /**
     * Computes the scale factor needed to resize the full map image to be as
     * large as possible, and cropping it vertically/horizontally so no empty
     * space exists if the container has different aspect from the image. This is
     * like CSS `background-size: cover` or SVG `aspectRatio xMidYMid slice`.
     */
    computeScale() {
        const img = this.imgSize;
        const svg = this.svgSize;
        // if (img === undefined || svg === undefined) {
        //   this.svgScale = this.initialScale = 1;
        // } else {
        this.svgScale = this.initialScale =
            Math.max(svg.width / img.width, svg.height / img.height);
        // }
    }
    /**
     * Determines the current size of the SVG element serving as the container
     * for the entire map.
     */
    computeSvgSize() {
        const root = this.root.querySelector('svg');
        const svg = root && root.getBoundingClientRect();
        if (svg !== null) {
            this.svgSize = new DOMRect(0, 0, svg.width, svg.height);
        }
    }
    /**
     * Sets the currently active element of the Map.
     *
     * @param el The Element that will be set as the active element.
     */
    _setActiveElement(el, shouldEmit = true) {
        if (!el || (this.activeElement && el.id === this.activeElement.id)) {
            return;
        }
        this.clearActiveElement();
        this.activeElement = el;
        this.activeElement.active = true;
        // Emit an event with the original (un-parsed) element as the new 'active'
        // element.  Don't emit an event if the flag is set.
        if (shouldEmit) {
            this.elementSelected.emit(this.elements[this.activeElement.id]);
        }
    }
    /**
     * Scales a given `Coordinate` by the inverse of the current `svgScale`.
     * Returns a new `Coordinate` with its values scaled and rounded.
     * @param c A `Coordinate` scale.
     */
    toSvgScale(c) {
        return c.clone().scale(1 / this.svgScale).round();
    }
    /**
     * Converts a given `Coordinate` object in screen coordinate space to SVG
     * coordinate space.
     * @param c A `Coordinate` to convert to SVG coordinate space.
     */
    toSvgSpace(c) {
        return Coordinate.difference(this.toSvgScale(c), this.toSvgScale(this.svgTransform));
    }
    hostData() {
        return {
            class: {
                'rl-map': true,
            },
        };
    }
    render() {
        const m = [
            this.svgScale, 0, 0, this.svgScale,
            this.svgTransform.x,
            this.svgTransform.y,
        ];
        const matrix = 'matrix(' + m.join(',') + ')';
        return (h("svg", { class: "rl-map__svg" },
            h("g", { class: "rl-map__transform", transform: matrix },
                h("g", { class: "rl-map__image-wrapper" },
                    h("image", { class: "rl-map__image", xlinkHref: this.mapImage !== undefined ? this.mapImage : undefined })),
                h("g", { class: "rl-map__elements" }, this.processedElements.map(el => el.render())))));
    }
    static get is() { return "rl-map"; }
    static get properties() { return {
        "activeElement": {
            "state": true
        },
        "clearActiveElement": {
            "method": true
        },
        "elements": {
            "type": "Any",
            "attr": "elements",
            "mutable": true,
            "watchCallbacks": ["onElementsChanged"]
        },
        "mapImage": {
            "type": String,
            "attr": "map-image",
            "watchCallbacks": ["onMapImageChanged"]
        },
        "maxScale": {
            "type": Number,
            "attr": "max-scale"
        },
        "minScale": {
            "type": Number,
            "attr": "min-scale"
        },
        "root": {
            "elementRef": true
        },
        "setActiveElement": {
            "method": true
        },
        "svgScale": {
            "state": true
        },
        "svgTransform": {
            "state": true
        }
    }; }
    static get events() { return [{
            "name": "elementSelected",
            "method": "elementSelected",
            "bubbles": true,
            "cancelable": true,
            "composed": true
        }, {
            "name": "elementDeselected",
            "method": "elementDeselected",
            "bubbles": true,
            "cancelable": true,
            "composed": true
        }, {
            "name": "mapRendered",
            "method": "mapRendered",
            "bubbles": true,
            "cancelable": true,
            "composed": true
        }]; }
    static get listeners() { return [{
            "name": "mousedown",
            "method": "onGestureDown",
            "passive": true
        }, {
            "name": "touchstart",
            "method": "onGestureDown",
            "passive": true
        }, {
            "name": "mousemove",
            "method": "onGestureMove",
            "passive": true
        }, {
            "name": "touchmove",
            "method": "onGestureMove",
            "passive": true
        }, {
            "name": "mouseup",
            "method": "onGestureUp",
            "passive": true
        }, {
            "name": "touchend",
            "method": "onGestureUp",
            "passive": true
        }, {
            "name": "mouseleave",
            "method": "onMouseLeave",
            "passive": true
        }, {
            "name": "window:resize",
            "method": "onResize",
            "passive": true
        }, {
            "name": "wheel",
            "method": "onWheel",
            "passive": true
        }, {
            "name": "keydown",
            "method": "onEnter"
        }]; }
    static get style() { return ".rl-map {\n  display: inline-block;\n  width: 100%;\n  height: 100%;\n  overflow: hidden; }\n  .rl-map__svg {\n    width: 100%;\n    height: 100%; }\n\n.rl-map-polygon {\n  fill: var(--rl-map-polygon-fill, rgba(0, 76, 155, 0.6));\n  stroke: var(--rl-map-polygon-fill, #004c9b);\n  stroke-width: 4; }\n  .rl-map-polygon:focus {\n    outline: none;\n    fill: var(--rl-map-element-focus-fill, rgba(255, 255, 83, 0.3)); }\n  .rl-map-polygon--activated {\n    stroke: var(--rl-map-element-active-stroke, #c7ab00);\n    stroke-width: 8; }\n  .rl-map-polygon:hover {\n    outline: none;\n    fill: var(--rl-map-element-focus-fill, rgba(255, 255, 83, 0.3));\n    cursor: pointer; }\n\n.rl-map-text {\n  font-family: var(--rl-map-element-font-family, \"sans\");\n  font-size: var(--rl-map-element-font-size, 24px);\n  -webkit-user-select: none;\n  -moz-user-select: none;\n  -ms-user-select: none;\n  user-select: none; }\n  .rl-map-text:hover {\n    cursor: pointer; }\n\n.rl-map-comp {\n  fill: red;\n  fill-rule: evenodd;\n  stroke: red;\n  stroke-width: 2; }\n  .rl-map-comp--available {\n    fill: green;\n    stroke: green; }\n\n.rl-map-outline {\n  fill: none;\n  stroke: var(--rl-map-element-active-stroke, #c7ab00);\n  stroke-width: 2; }\n\n.rl-map-control {\n  stroke: black;\n  fill: rgba(255, 255, 255, 0.6);\n  stroke-width: 2; }\n  .rl-map-control:hover {\n    fill: rgba(192, 192, 192, 0.6);\n    cursor: pointer; }\n\n.rl-map-midpoint {\n  stroke: rgba(0, 0, 0, 0.6);\n  fill: rgba(255, 255, 255, 0.3);\n  stroke-width: 2; }\n  .rl-map-midpoint:hover {\n    fill: rgba(192, 192, 192, 0.3);\n    cursor: pointer; }\n\n.rl-map-marker:focus rect {\n  outline: none;\n  fill: var(--rl-map-element-focus-fill, rgba(255, 255, 83, 0.3)); }\n\n.rl-map-marker--clickable:hover rect {\n  outline: none;\n  fill: var(--rl-map-element-focus-fill, rgba(255, 255, 83, 0.3));\n  cursor: pointer; }\n\n.rl-map-rect {\n  fill: none;\n  stroke: none; }\n  .rl-map-rect--activated {\n    stroke: var(--rl-map-element-active-stroke, #c7ab00);\n    stroke-width: 8; }"; }
}

export { RLMap as RlMap };
