import '../../stencil.core';
import { EventEmitter } from '../../stencil.core';
import { MapMarker } from '../../classes/map-marker';
import { MapPolygon } from '../../classes/map-polygon';
import { MapElementData } from '../../interface';
import { Coordinate } from '../../utils/coordinate';
export declare class RLMapEditor {
    /**
     * The array of MapElements currently being displayed.  Created from the
     * `elements` Prop with additional internal information added.
     */
    private processedElements;
    private targetElement;
    private targetControl;
    private imgSize;
    private initialScale;
    private last;
    private limits;
    private resizeDebounce;
    private start;
    private state;
    private svgSize;
    root: HTMLStencilElement;
    /**
     * The currently active element.
     */
    activeElement: MapMarker | MapPolygon | undefined;
    /**
     * The factor by which the Map contents are changed to fit within the SVG.
     */
    svgScale: number;
    /**
     * The x,y translation applied to move the map contents within the bounds of
     * the SVG.
     */
    svgTransform: Coordinate;
    /**
     * An array of the elements that will be displayed on the Map.
     */
    elements?: MapElementData[];
    /**
     * The image being displayed as the base of the map.
     */
    mapImage?: string;
    onMapImageChanged(): void;
    /**
     *  The maximum scale factor.
     */
    maxScale: number;
    /**
     * The minimum scale factor.
     */
    minScale: number;
    /**
     * An even fired when the user selects a `MapElement`. The clicked element's
     * `id` will be passed in the event details.
     */
    elementSelected: EventEmitter<MapElementData>;
    /**
     * An event fired when a `MapElement` is updated (moved or changes shape).
     * The event details contains the `MapElement` that was moved.
     */
    elementUpdated: EventEmitter<MapElementData>;
    /**
     * An event fired when a new `MapElement` is created. The event details
     * contains the `MapElement` that was created.
     */
    elementCreated: EventEmitter<MapElementData>;
    /**
     * An event fired when the user deselects a `MapElement`.
     */
    elementDeselected: EventEmitter<undefined>;
    /**
     * An event fired when one of the `MapElements` on this map is deleted.
     */
    elementDeleted: EventEmitter<MapElementData>;
    /**
     * An event fired when one of the `MapElement`s on the map is double clicked.
     */
    elementDoubleClicked: EventEmitter<MapElementData>;
    /**
     * Handle when the list of specified elements changes. The event details
     * contains the `MapElement` that was deleted.
     */
    onElementsChanged(): void;
    componentDidLoad(): void;
    /**
     * Handles when the user triggers a touch/down event to start interaction with
     * the map.
     * @param e The triggering event.
     */
    onGestureDown(e: Event): void;
    /**
     * Handles when the user triggers a movement event.
     * @param e The triggering event.
     */
    onGestureMove(e: Event): void;
    /**
     * Handles when the user ends their interaction with the Map.
     * @param e The triggering event.
     */
    onGestureUp(e: Event): void;
    /**
     * Handles when the user leaves the bounds of the Map entirely.
     * @param e The triggering event.
     */
    onMouseLeave(e: Event): void;
    onDoubleClick(e: Event): void;
    /**
     * Handles when the window is resized.
     */
    onResize(): void;
    /**
     * Handles when the user performs a mouse wheen event.  The Map will be zoomed
     * in or out depending on the direction of the wheel.
     *
     * @param e The triggering event.
     */
    onWheel(e: WheelEvent): void;
    /**
     * Handle when the user pressses the Enter key.
     * @param e The triggering event
     */
    onEnter(e: KeyboardEvent): void;
    /**
     * Add a new `MapRegion` to the map. Calling this method starts the process.
     * The user must then click numerous times on the map to add points.  Only
     * when clicking again on the original point is the region added.
     */
    addRegion(): void;
    /**
     * Add a new `MapPoint` element to the map.  Calling this method starts the
     * process.  The user must then click somewhere on the map to add the point.
     */
    addPoint(): void;
    /**
     * Cancels the current action and returns the map to its default state, ready
     * for futher action.
     */
    cancelAction(): void;
    /**
     * Removes the currently active element.  If no element is selected when
     * this method is called, it has no effect.
     */
    deleteRegion(): void;
    /**
     * Sets the element with the specified ID to active.
     *
     * @param id The ID of the element to set as active.
     */
    setActiveElement(id: number): void;
    private mapElementFromParsedElement;
    /**
     * Clears the currently active element.
     */
    private _clearActiveElement;
    /**
     * Updates the limits used to restrict the how far the SVG transform can be
     * moved keeping the map from being panning beyond the edge of the image.
     */
    private updateLimits;
    /**
     * Computes the scale factor needed to resize the full map image to be as
     * large as possible, and cropping it vertically/horizontally so no empty
     * space exists if the container has different aspect from the image. This is
     * like CSS `background-size: cover` or SVG `aspectRatio xMidYMid slice`.
     */
    private computeScale;
    protected computeSvgSize(): void;
    /**
     * Sets the currently active element of the Map.
     *
     * @param el The Element that will be set as the active element.
     */
    private _setActiveElement;
    /**
     * Scales a given `Coordinate` by the inverse of the current `svgScale`.
     * Returns a new `Coordinate` with its values scaled and rounded.
     * @param c A `Coordinate` scale.
     */
    private toSvgScale;
    /**
     * Converts a given `Coordinate` object in screen coordinate space to SVG
     * coordinate space.
     * @param c A `Coordinate` to convert to SVG coordinate space.
     */
    private toSvgSpace;
    hostData(): {
        class: {
            'rl-map': boolean;
            'rl-map-editor': boolean;
        };
    };
    render(): JSX.Element;
}
