import '../../stencil.core';
import { EventEmitter } from '../../stencil.core';
import { MapMarker } from '../../classes/map-marker';
import { MapPolygon } from '../../classes/map-polygon';
import { MapElementDataMap } from '../../interface';
import { Coordinate } from '../../utils/coordinate';
export declare class RLMap {
    /**
     * The array of MapElements currently being displayed.  Created from the
     * `elements` Prop with additional internal information added.
     */
    private processedElements;
    private targetElement;
    private imgSize;
    private initialScale;
    private last;
    private limits;
    private resizeDebounce;
    private start;
    private state;
    private svgSize;
    root: HTMLElement;
    /**
     * The currently active element.
     */
    activeElement: MapMarker | MapPolygon | undefined;
    /**
     * The factor by which the Map contents are changed to fit within the SVG.
     */
    svgScale: number;
    /**
     * The x,y translation applied to move the map contents within the bounds of
     * the SVG.
     */
    svgTransform: Coordinate;
    /**
     * An image that will be displayed on the Map.
     */
    mapImage?: string;
    /**
     * Handles when the map Image is loaded/updated.
     */
    onMapImageChanged(): void;
    /**
     *  The maximum scale factor.
     */
    maxScale: number;
    /**
     * The minimum scale factor.
     */
    minScale: number;
    /**
     * An event fired when the user selects a MapElement. The clicked element
     * will be passed as the event parameter.
     */
    elementSelected: EventEmitter;
    /**
     * An event fired when the user deselects the selected MapElement. The clicked
     * element will be passed as the event parameter.
     */
    elementDeselected: EventEmitter;
    /**
     * An event fired when the map floorplan is updated.
     */
    mapRendered: EventEmitter;
    /**
     * An array of the elements that will be displayed on the Map.
     */
    elements: MapElementDataMap;
    /**
     * Handle when the list of specified elements changes.
     */
    onElementsChanged(): void;
    componentDidLoad(): void;
    componentDidUpdate(): void;
    /**
     * Handles when the user triggers a touch/down event to start interaction with
     * the map.
     * @param e The triggering event.
     */
    onGestureDown(e: Event): void;
    /**
     * Handles when the user triggers a movement event.
     * @param e The triggering event.
     */
    onGestureMove(e: Event): void;
    /**
     * Handles when the user ends their interaction with the Map.
     * @param e The triggering event.
     */
    onGestureUp(e: Event): void;
    /**
     * Handles when the user leaves the bounds of the Map entirely.
     * @param e The triggering event.
     */
    onMouseLeave(e: Event): void;
    /**
     * Handles when the user performs a resize action.
     */
    onResize(): void;
    /**
     * Handles when the user performs a mouse wheen event.  The Map will be zoomed
     * in or out depending on the direction of the wheel.
     *
     * @param e The triggering event.
     */
    onWheel(e: WheelEvent): void;
    onEnter(e: KeyboardEvent): void;
    /**
     * Clears the currently active element.
     */
    clearActiveElement(): void;
    /**
     * Sets the element with the specified ID to active.
     *
     * @param id The ID of the element to set as active.
     */
    setActiveElement(id: number): void;
    /**
     * Clears the currently active element.
     */
    private _clearActiveElement;
    /**
     * Updates the limits used to restrict the how far the SVG transform can be
     * moved keeping the map from being panning beyond the edge of the image.
     */
    private computeLimits;
    /**
     * Computes the scale factor needed to resize the full map image to be as
     * large as possible, and cropping it vertically/horizontally so no empty
     * space exists if the container has different aspect from the image. This is
     * like CSS `background-size: cover` or SVG `aspectRatio xMidYMid slice`.
     */
    private computeScale;
    /**
     * Determines the current size of the SVG element serving as the container
     * for the entire map.
     */
    private computeSvgSize;
    /**
     * Sets the currently active element of the Map.
     *
     * @param el The Element that will be set as the active element.
     */
    private _setActiveElement;
    /**
     * Scales a given `Coordinate` by the inverse of the current `svgScale`.
     * Returns a new `Coordinate` with its values scaled and rounded.
     * @param c A `Coordinate` scale.
     */
    private toSvgScale;
    /**
     * Converts a given `Coordinate` object in screen coordinate space to SVG
     * coordinate space.
     * @param c A `Coordinate` to convert to SVG coordinate space.
     */
    private toSvgSpace;
    hostData(): {
        class: {
            'rl-map': boolean;
        };
    };
    render(): JSX.Element;
}
